@@ -240,8 +240,7 @@ client_deliver_delete(Client *c)
 
   if (c->has_ping_protocol && c->pings_pending == -1) 
     {
-      c->pings_pending = 0;
-      w->n_active_ping_clients++;
+      ewmh_ping_client_start (c);
     }
 
   if (found)
@@ -344,6 +343,7 @@ client_set_focus(Client *c)
   return False;
 }
 
+
 void
 client_get_transient_list(Wm *w, MBList **list, Client *c)
 {
@@ -384,14 +384,37 @@ client_get_transient_list(Wm *w, MBList **list, Client *c)
 
 	      while (trans != NULL && trans != c)
 		trans = trans->trans;
-
+	      
 	      if (trans == c)
-		list_add(list, NULL, 0, p);
+		{
+		  list_add(list, NULL, 0, p);
+		}
+	      else if (p->trans 
+		       && c->win_group 
+		       && (c->type == MBCLIENT_TYPE_APP
+			   || c->type == MBCLIENT_TYPE_DESKTOP))
+		{
+		  /* Handle window groups and transiency. 
+		   * App windows with matchbox window groups
+                   * 'share' transients  
+		  */
+		  trans = p->trans;
+
+		  while (trans->trans != NULL)
+		    trans = trans->trans;
+
+		  if ((trans->type == MBCLIENT_TYPE_APP
+		       || trans->type == MBCLIENT_TYPE_DESKTOP)
+		      && trans->win_group == c->win_group) 
+		    list_add(list, NULL, 0, p);
+		}
+
 	    }
 	}
     }
 }
 
+
 static Client*
 client_get_highest_transient_recurse (Client *c, 
 				      int     client_flags, 
