@@ -81,6 +81,18 @@ dialog_client_get_offsets(Client *c, int *e, int *s, int *w)
       return;
     }
 
+  if (c->flags & CLIENT_BORDERS_ONLY_FLAG
+      && theme_has_borders_only_decor(c->wm->mbtheme))
+    {
+      *s = theme_frame_defined_height_get(c->wm->mbtheme, 
+					  FRAME_DIALOG_NT_SOUTH);
+      *e = theme_frame_defined_width_get(c->wm->mbtheme, 
+					 FRAME_DIALOG_NT_EAST );
+      *w = theme_frame_defined_width_get(c->wm->mbtheme, 
+					 FRAME_DIALOG_NT_WEST );
+      return;
+    }
+
   if (c->flags & CLIENT_HAS_URGENCY_FLAG
       && theme_has_message_decor(c->wm->mbtheme))
     {
@@ -199,10 +211,14 @@ dialog_client_title_height(Client *c)
       return theme_frame_defined_height_get(c->wm->mbtheme, FRAME_MSG);
     }
   
-  if (c->flags & CLIENT_BORDERS_ONLY_FLAG
-      && theme_has_frame_type_defined(c->wm->mbtheme, FRAME_DIALOG_NORTH))    
-    return theme_frame_defined_height_get(c->wm->mbtheme, FRAME_DIALOG_NORTH);
-  
+  if (c->flags & CLIENT_BORDERS_ONLY_FLAG)
+    {
+      if (theme_has_borders_only_decor (c->wm->mbtheme))
+	return theme_frame_defined_height_get(c->wm->mbtheme, FRAME_DIALOG_NT_NORTH);
+      else if (theme_has_frame_type_defined(c->wm->mbtheme, FRAME_DIALOG_NORTH))
+	return theme_frame_defined_height_get(c->wm->mbtheme, FRAME_DIALOG_NORTH);
+    }  
+
   return theme_frame_defined_height_get(c->wm->mbtheme, FRAME_DIALOG);
 }
 
@@ -225,14 +241,14 @@ dialog_client_show(Client *c)
 	  client_set_state(c, NormalState);
 	  c->flags &= ~CLIENT_IS_MINIMIZED;
 
-	   /* Make sure any transients are un minimized too */
-	   stack_enumerate(w, p)
-	     if (p->trans == c)
-	       {
-		 p->show(p);
-		 XMapSubwindows(w->dpy, p->frame);
-		 XMapWindow(w->dpy, p->frame);
-	       }
+	  /* Make sure any transients are un minimized too */
+	  stack_enumerate(w, p)
+	    if (p->trans == c)
+	      {
+		p->show(p);
+		XMapSubwindows(w->dpy, p->frame);
+		XMapWindow(w->dpy, p->frame);
+	      }
 	}
 
       if (c->win_modal_blocker)
@@ -302,6 +318,20 @@ dialog_client_show(Client *c)
 	      stack_move_above_client(c, highest_client);
 	    }
 	}
+
+      if (lowest_trans->win_group 
+	  && (lowest_trans->type == MBCLIENT_TYPE_DESKTOP
+	      || lowest_trans->type == MBCLIENT_TYPE_APP))
+	{
+	  Client *visible;
+
+	  visible = wm_get_visible_main_client(w);
+
+	  if (visible 
+	      && visible != lowest_trans 
+	      && visible->win_group == lowest_trans->win_group)
+	    stack_move_above_client(c, visible);
+	}
     }
   else
     stack_move_top(c);
@@ -795,11 +825,18 @@ dialog_client_redraw(Client *c, Bool use_cache)
       frame_ref_west  = FRAME_MSG_WEST;
       frame_ref_south = FRAME_MSG_SOUTH;
     }
+  else if (c->flags & CLIENT_BORDERS_ONLY_FLAG
+	   && theme_has_borders_only_decor(c->wm->mbtheme))
+    {
+      frame_ref_top   = FRAME_DIALOG_NT_NORTH;
+      frame_ref_east  = FRAME_DIALOG_NT_EAST;
+      frame_ref_west  = FRAME_DIALOG_NT_WEST;
+      frame_ref_south = FRAME_DIALOG_NT_SOUTH;
+    }
 
   dbg("%s() c->width : %i , offset_east : %i, offset_west : %i\n",
       __func__, c->width, offset_east, offset_west );
 
-
   is_shaped = theme_frame_wants_shaped_window( c->wm->mbtheme, frame_ref_top);
 
   if (is_shaped) client_init_backing_mask(c, total_w, c->height, 
